# Try rolling window null
# Mark Peterson
# 2015/Dec/01

setwd("~/Documents/Juniata/research/grantFish/updateonbrooktroutresearchthissemester/smoothedFST/")

fullData <- read.csv("Fig3_data_forMark.csv")

head(fullData)

table(fullData$Chr)

plot(BP ~ Position, data = fullData)
# ok, so each chr has its own size as "BP" and
# "Position" is the combined

# First, calculate rolling windows for actual data

# separating by chr to ease handling.
# Should I really do this for the chr with only 1 or 2 positions? Including for now.

sigma <- 150000
# thisChr <- split(fullData, fullData$Chr)[[1]]
smoothedFst <- lapply(split(fullData, fullData$Chr), function(thisChr){
  # x <- as.character(thisChr[1,])
  # names(x) <- names(thisChr)
  apply(thisChr, 1, function(x){
    thisPos <- as.numeric(x["BP"])
    inRange <- 
      thisChr$BP >= (thisPos - sigma*3) & 
      thisChr$BP <= (thisPos + sigma*3)
    myWts <- dnorm(thisChr$BP[inRange]
                   , mean = thisPos
                   , sd = sigma)
    myWts <- myWts/sum(myWts)
    return(sum(myWts*thisChr$AMOVA.Fst[inRange]))
  })
})

plot(unlist(smoothedFst))
plot(unlist(smoothedFst) ~ fullData$Smoothed.AMOVA.Fst)
# where are divergences coming?
# ID chrom borders
firstPos <- fullData$Position[!duplicated(fullData$Chr)]
# firstPos
# Color pts with 450000 of those
chrEnds <- data.frame(end = firstPos - sigma*3
                      , begin = firstPos + sigma*3)
inEnd <- apply(apply(chrEnds,1, function(x){
  fullData$Position > x[1] & fullData$Position < x[2]
}), 1, any)

table(inEnd)
plot(unlist(smoothedFst) ~ fullData$Smoothed.AMOVA.Fst
     , col = inEnd+1)
# Not caused by chromosome ends

plot(unlist(smoothedFst) ~ fullData$Smoothed.AMOVA.Fst)
abline(0.1, 1, lty = 3)
abline(-0.1, 1, lty = 3)

# I'm going with "Close enough"




randomSmoothedFst <- lapply(1:1000,function(idx){
  
  randData <- fullData
  randData$AMOVA.Fst <- sample(fullData$AMOVA.Fst)
  tempSmoothedFst <- lapply(split(randData, randData$Chr), function(thisChr){
    # x <- as.character(thisChr[1,])
    # names(x) <- names(thisChr)
    apply(thisChr, 1, function(x){
      thisPos <- as.numeric(x["BP"])
      inRange <- 
        thisChr$BP >= (thisPos - sigma*3) & 
        thisChr$BP <= (thisPos + sigma*3)
      myWts <- dnorm(thisChr$BP[inRange]
                     , mean = thisPos
                     , sd = sigma)
      myWts <- myWts/sum(myWts)
      return(sum(myWts*thisChr$AMOVA.Fst[inRange]))
    })
  })
  return(unlist(tempSmoothedFst))
})


hist(unlist(randomSmoothedFst))
hist(unlist(smoothedFst))

# VERY Close
# Want to try my own binning
myBreaks <- seq(0,1,0.05)
myLabs <- myBreaks[-1] - 0.025
binnedVals <-
  cbind(
    Random = table(cut(unlist(randomSmoothedFst)
                       , myBreaks, myLabs))
    , Observed = table(cut(unlist(smoothedFst)
                           , myBreaks, myLabs))
  )

png("distributionOfRandomSmoothedFst.png")
barplot(t(prop.table(binnedVals, 2)), beside = TRUE
        , legend.text = TRUE
        , las = 2
        , ylab = "Proportion")
title(xlab = expression(paste("Smoothed ", F[ST], " (Bin Center)"))
      , line = 4)
dev.off()

boxplot(unlist(randomSmoothedFst), unlist(smoothedFst)
        , names = c("Random","Observed"))


plot(unlist(smoothedFst))
plot(fullData$Smoothed.AMOVA.Fst)

toPlot <- lapply(names(smoothedFst), function(x){
  data.frame(pos = fullData$BP[fullData$Chr == x]
             , smoothFst = smoothedFst[[x]])
})
names(toPlot) <- names(smoothedFst)

png("observedSmoothFst.png", width = 800, height = 400)
par(mar = c(0.1,0.1,2,0.1)
    , mfrow = c(1,length(smoothedFst))
    , oma = c(2.5,5,1,1)
    )
catch <- lapply(names(toPlot), function(x){
  plot(toPlot[[x]]
       , ylim = range(unlist(smoothedFst))
       , yaxt = "n"
       , ylab = ""
       , xlab = ""
       , xaxt = "n"
       , main = substr(x,14 ,15) 
       , cex.main = 0.75
       )
  if(x == names(toPlot)[1]){
    axis(2, xpd = TRUE, las = 2, cex.axis =1.5)
  }
})
title(ylab = expression(paste("Smoothed ", F[ST]))
      , outer = TRUE
      , line = 3
      , cex.lab = 1.5)
title(xlab = "Position"
      , outer = TRUE
      , line = 0.75
      , cex.lab = 1.5)
dev.off()
